from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.create_note_def_type import CreateNoteDefType
from ..types import UNSET, Unset

T = TypeVar("T", bound="CreateNoteDef")


@attr.s(auto_attribs=True)
class CreateNoteDef:
    """
    Attributes:
        parent_note_id (str):  Example: evnnmvHTCgIn.
        title (str):
        type (CreateNoteDefType):
        content (str):
        mime (Union[Unset, str]): this needs to be specified only for note types 'code', 'file', 'image'. Example:
            application/json.
        note_position (Union[Unset, int]): Position of the note in the parent. Normal ordering is 10, 20, 30 ...  So if
            you want to create a note on the first position, use e.g. 5, for second position 15, for last e.g. 1000000
        prefix (Union[Unset, str]): Prefix is branch (placement) specific title prefix for the note.  Let's say you have
            your note placed into two different places in the tree,  but you want to change the title a bit in one of the
            placements. For this you can use prefix.
        is_expanded (Union[Unset, bool]): true if this note (as a folder) should appear expanded
        note_id (Union[Unset, str]):  Example: evnnmvHTCgIn.
        branch_id (Union[Unset, str]):  Example: evnnmvHTCgIn.
    """

    parent_note_id: str
    title: str
    type: CreateNoteDefType
    content: str
    mime: Union[Unset, str] = UNSET
    note_position: Union[Unset, int] = UNSET
    prefix: Union[Unset, str] = UNSET
    is_expanded: Union[Unset, bool] = UNSET
    note_id: Union[Unset, str] = UNSET
    branch_id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        parent_note_id = self.parent_note_id
        title = self.title
        type = self.type.value

        content = self.content
        mime = self.mime
        note_position = self.note_position
        prefix = self.prefix
        is_expanded = self.is_expanded
        note_id = self.note_id
        branch_id = self.branch_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "parentNoteId": parent_note_id,
                "title": title,
                "type": type,
                "content": content,
            }
        )
        if mime is not UNSET:
            field_dict["mime"] = mime
        if note_position is not UNSET:
            field_dict["notePosition"] = note_position
        if prefix is not UNSET:
            field_dict["prefix"] = prefix
        if is_expanded is not UNSET:
            field_dict["isExpanded"] = is_expanded
        if note_id is not UNSET:
            field_dict["noteId"] = note_id
        if branch_id is not UNSET:
            field_dict["branchId"] = branch_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        parent_note_id = d.pop("parentNoteId")

        title = d.pop("title")

        type = CreateNoteDefType(d.pop("type"))

        content = d.pop("content")

        mime = d.pop("mime", UNSET)

        note_position = d.pop("notePosition", UNSET)

        prefix = d.pop("prefix", UNSET)

        is_expanded = d.pop("isExpanded", UNSET)

        note_id = d.pop("noteId", UNSET)

        branch_id = d.pop("branchId", UNSET)

        create_note_def = cls(
            parent_note_id=parent_note_id,
            title=title,
            type=type,
            content=content,
            mime=mime,
            note_position=note_position,
            prefix=prefix,
            is_expanded=is_expanded,
            note_id=note_id,
            branch_id=branch_id,
        )

        create_note_def.additional_properties = d
        return create_note_def

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from http import HTTPStatus
from typing import Any, Dict, Optional

import httpx

from ... import errors
from ...client import Client
from ...models.note import Note
from ...types import Response


def _get_kwargs(
    date: datetime.date,
    *,
    client: Client,
) -> Dict[str, Any]:
    url = "{}/inbox/{date}".format(client.base_url, date=date)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "method": "get",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
    }


def _parse_response(*, client: Client, response: httpx.Response) -> Optional[Note]:
    if response.status_code == HTTPStatus.OK:
        response_200 = Note.from_dict(response.json())

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(f"Unexpected status code: {response.status_code}")
    else:
        return None


def _build_response(*, client: Client, response: httpx.Response) -> Response[Note]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    date: datetime.date,
    *,
    client: Client,
) -> Response[Note]:
    r"""returns an \"inbox\" note, into which note can be created. Date will be used depending on whether
    the inbox is a fixed note (identified with #inbox label) or a day note in a journal.

    Args:
        date (datetime.date):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Note]
    """

    kwargs = _get_kwargs(
        date=date,
        client=client,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    date: datetime.date,
    *,
    client: Client,
) -> Optional[Note]:
    r"""returns an \"inbox\" note, into which note can be created. Date will be used depending on whether
    the inbox is a fixed note (identified with #inbox label) or a day note in a journal.

    Args:
        date (datetime.date):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Note]
    """

    return sync_detailed(
        date=date,
        client=client,
    ).parsed


async def asyncio_detailed(
    date: datetime.date,
    *,
    client: Client,
) -> Response[Note]:
    r"""returns an \"inbox\" note, into which note can be created. Date will be used depending on whether
    the inbox is a fixed note (identified with #inbox label) or a day note in a journal.

    Args:
        date (datetime.date):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Note]
    """

    kwargs = _get_kwargs(
        date=date,
        client=client,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    date: datetime.date,
    *,
    client: Client,
) -> Optional[Note]:
    r"""returns an \"inbox\" note, into which note can be created. Date will be used depending on whether
    the inbox is a fixed note (identified with #inbox label) or a day note in a journal.

    Args:
        date (datetime.date):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Note]
    """

    return (
        await asyncio_detailed(
            date=date,
            client=client,
        )
    ).parsed



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="AppInfo")


@attr.s(auto_attribs=True)
class AppInfo:
    """
    Attributes:
        app_version (Union[Unset, str]): Trilium version Example: 0.50.2.
        db_version (Union[Unset, int]): DB version Example: 194.
        sync_version (Union[Unset, int]): Sync protocol version Example: 25.
        build_date (Union[Unset, datetime.datetime]): build date Example: 2022-02-09 22:52:36+01:00.
        build_revision (Union[Unset, str]): git build revision Example: 23daaa2387a0655685377f0a541d154aeec2aae8.
        data_directory (Union[Unset, str]): data directory where Trilium stores files Example: /home/user/data.
        clipper_protocol_version (Union[Unset, str]): version of the supported Trilium Web Clipper protocol Example:
            1.0.
        utc_date_time (Union[Unset, str]): current UTC date time Example: 2022-03-07 21:54:25.277000+00:00.
    """

    app_version: Union[Unset, str] = UNSET
    db_version: Union[Unset, int] = UNSET
    sync_version: Union[Unset, int] = UNSET
    build_date: Union[Unset, datetime.datetime] = UNSET
    build_revision: Union[Unset, str] = UNSET
    data_directory: Union[Unset, str] = UNSET
    clipper_protocol_version: Union[Unset, str] = UNSET
    utc_date_time: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        app_version = self.app_version
        db_version = self.db_version
        sync_version = self.sync_version
        build_date: Union[Unset, str] = UNSET
        if not isinstance(self.build_date, Unset):
            build_date = self.build_date.isoformat()

        build_revision = self.build_revision
        data_directory = self.data_directory
        clipper_protocol_version = self.clipper_protocol_version
        utc_date_time = self.utc_date_time

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if app_version is not UNSET:
            field_dict["appVersion"] = app_version
        if db_version is not UNSET:
            field_dict["dbVersion"] = db_version
        if sync_version is not UNSET:
            field_dict["syncVersion"] = sync_version
        if build_date is not UNSET:
            field_dict["buildDate"] = build_date
        if build_revision is not UNSET:
            field_dict["buildRevision"] = build_revision
        if data_directory is not UNSET:
            field_dict["dataDirectory"] = data_directory
        if clipper_protocol_version is not UNSET:
            field_dict["clipperProtocolVersion"] = clipper_protocol_version
        if utc_date_time is not UNSET:
            field_dict["utcDateTime"] = utc_date_time

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        app_version = d.pop("appVersion", UNSET)

        db_version = d.pop("dbVersion", UNSET)

        sync_version = d.pop("syncVersion", UNSET)

        _build_date = d.pop("buildDate", UNSET)
        build_date: Union[Unset, datetime.datetime]
        if isinstance(_build_date, Unset):
            build_date = UNSET
        else:
            build_date = isoparse(_build_date)

        build_revision = d.pop("buildRevision", UNSET)

        data_directory = d.pop("dataDirectory", UNSET)

        clipper_protocol_version = d.pop("clipperProtocolVersion", UNSET)

        utc_date_time = d.pop("utcDateTime", UNSET)

        app_info = cls(
            app_version=app_version,
            db_version=db_version,
            sync_version=sync_version,
            build_date=build_date,
            build_revision=build_revision,
            data_directory=data_directory,
            clipper_protocol_version=clipper_protocol_version,
            utc_date_time=utc_date_time,
        )

        app_info.additional_properties = d
        return app_info

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import Client
from ...models.export_note_subtree_format import ExportNoteSubtreeFormat
from ...types import UNSET, Response, Unset


def _get_kwargs(
    note_id: str,
    *,
    client: Client,
    format_: Union[Unset, None, ExportNoteSubtreeFormat] = ExportNoteSubtreeFormat.HTML,
) -> Dict[str, Any]:
    url = "{}/notes/{noteId}/export".format(client.base_url, noteId=note_id)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    json_format_: Union[Unset, None, str] = UNSET
    if not isinstance(format_, Unset):
        json_format_ = format_.value if format_ else None

    params["format"] = json_format_

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(*, client: Client, response: httpx.Response) -> Optional[Any]:
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(f"Unexpected status code: {response.status_code}")
    else:
        return None


def _build_response(*, client: Client, response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    note_id: str,
    *,
    client: Client,
    format_: Union[Unset, None, ExportNoteSubtreeFormat] = ExportNoteSubtreeFormat.HTML,
) -> Response[Any]:
    r"""Exports ZIP file export of a given note subtree. To export whole document, use \"root\" for noteId

    Args:
        note_id (str):  Example: evnnmvHTCgIn.
        format_ (Union[Unset, None, ExportNoteSubtreeFormat]):  Default:
            ExportNoteSubtreeFormat.HTML.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        note_id=note_id,
        client=client,
        format_=format_,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(client=client, response=response)


async def asyncio_detailed(
    note_id: str,
    *,
    client: Client,
    format_: Union[Unset, None, ExportNoteSubtreeFormat] = ExportNoteSubtreeFormat.HTML,
) -> Response[Any]:
    r"""Exports ZIP file export of a given note subtree. To export whole document, use \"root\" for noteId

    Args:
        note_id (str):  Example: evnnmvHTCgIn.
        format_ (Union[Unset, None, ExportNoteSubtreeFormat]):  Default:
            ExportNoteSubtreeFormat.HTML.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        note_id=note_id,
        client=client,
        format_=format_,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(client=client, response=response)



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="Branch")


@attr.s(auto_attribs=True)
class Branch:
    """Branch places the note into the tree, it represents the relationship between a parent note and child note

    Attributes:
        note_id (str):  Example: evnnmvHTCgIn.
        parent_note_id (str):  Example: evnnmvHTCgIn.
        branch_id (Union[Unset, str]):  Example: evnnmvHTCgIn.
        prefix (Union[Unset, str]):
        note_position (Union[Unset, int]):
        is_expanded (Union[Unset, bool]):
        utc_date_modified (Union[Unset, str]):  Example: 2021-12-31 19:18:11.939000+00:00.
    """

    note_id: str
    parent_note_id: str
    branch_id: Union[Unset, str] = UNSET
    prefix: Union[Unset, str] = UNSET
    note_position: Union[Unset, int] = UNSET
    is_expanded: Union[Unset, bool] = UNSET
    utc_date_modified: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        note_id = self.note_id
        parent_note_id = self.parent_note_id
        branch_id = self.branch_id
        prefix = self.prefix
        note_position = self.note_position
        is_expanded = self.is_expanded
        utc_date_modified = self.utc_date_modified

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "noteId": note_id,
                "parentNoteId": parent_note_id,
            }
        )
        if branch_id is not UNSET:
            field_dict["branchId"] = branch_id
        if prefix is not UNSET:
            field_dict["prefix"] = prefix
        if note_position is not UNSET:
            field_dict["notePosition"] = note_position
        if is_expanded is not UNSET:
            field_dict["isExpanded"] = is_expanded
        if utc_date_modified is not UNSET:
            field_dict["utcDateModified"] = utc_date_modified

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        note_id = d.pop("noteId")

        parent_note_id = d.pop("parentNoteId")

        branch_id = d.pop("branchId", UNSET)

        prefix = d.pop("prefix", UNSET)

        note_position = d.pop("notePosition", UNSET)

        is_expanded = d.pop("isExpanded", UNSET)

        utc_date_modified = d.pop("utcDateModified", UNSET)

        branch = cls(
            note_id=note_id,
            parent_note_id=parent_note_id,
            branch_id=branch_id,
            prefix=prefix,
            note_position=note_position,
            is_expanded=is_expanded,
            utc_date_modified=utc_date_modified,
        )

        branch.additional_properties = d
        return branch

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.attribute_type import AttributeType
from ..types import UNSET, Unset

T = TypeVar("T", bound="Attribute")


@attr.s(auto_attribs=True)
class Attribute:
    """Attribute (Label, Relation) is a key-value record attached to a note.

    Attributes:
        note_id (str):  Example: evnnmvHTCgIn.
        attribute_id (Union[Unset, str]):  Example: evnnmvHTCgIn.
        type (Union[Unset, AttributeType]):
        name (Union[Unset, str]):  Example: shareCss.
        value (Union[Unset, str]):
        position (Union[Unset, int]):
        is_inheritable (Union[Unset, bool]):
        utc_date_modified (Union[Unset, str]):  Example: 2021-12-31 19:18:11.939000+00:00.
    """

    note_id: str
    attribute_id: Union[Unset, str] = UNSET
    type: Union[Unset, AttributeType] = UNSET
    name: Union[Unset, str] = UNSET
    value: Union[Unset, str] = UNSET
    position: Union[Unset, int] = UNSET
    is_inheritable: Union[Unset, bool] = UNSET
    utc_date_modified: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        note_id = self.note_id
        attribute_id = self.attribute_id
        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        name = self.name
        value = self.value
        position = self.position
        is_inheritable = self.is_inheritable
        utc_date_modified = self.utc_date_modified

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "noteId": note_id,
            }
        )
        if attribute_id is not UNSET:
            field_dict["attributeId"] = attribute_id
        if type is not UNSET:
            field_dict["type"] = type
        if name is not UNSET:
            field_dict["name"] = name
        if value is not UNSET:
            field_dict["value"] = value
        if position is not UNSET:
            field_dict["position"] = position
        if is_inheritable is not UNSET:
            field_dict["isInheritable"] = is_inheritable
        if utc_date_modified is not UNSET:
            field_dict["utcDateModified"] = utc_date_modified

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        note_id = d.pop("noteId")

        attribute_id = d.pop("attributeId", UNSET)

        _type = d.pop("type", UNSET)
        type: Union[Unset, AttributeType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = AttributeType(_type)

        name = d.pop("name", UNSET)

        value = d.pop("value", UNSET)

        position = d.pop("position", UNSET)

        is_inheritable = d.pop("isInheritable", UNSET)

        utc_date_modified = d.pop("utcDateModified", UNSET)

        attribute = cls(
            note_id=note_id,
            attribute_id=attribute_id,
            type=type,
            name=name,
            value=value,
            position=position,
            is_inheritable=is_inheritable,
            utc_date_modified=utc_date_modified,
        )

        attribute.additional_properties = d
        return attribute

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.note_type import NoteType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.attribute import Attribute


T = TypeVar("T", bound="Note")


@attr.s(auto_attribs=True)
class Note:
    """
    Attributes:
        note_id (Union[Unset, str]):  Example: evnnmvHTCgIn.
        title (Union[Unset, str]):
        type (Union[Unset, NoteType]):
        mime (Union[Unset, str]):
        is_protected (Union[Unset, bool]):
        attributes (Union[Unset, List['Attribute']]):
        parent_note_ids (Union[Unset, List[str]]):
        child_note_ids (Union[Unset, List[str]]):
        parent_branch_ids (Union[Unset, List[str]]):
        child_branch_ids (Union[Unset, List[str]]):
        date_created (Union[Unset, str]):  Example: 2021-12-31 20:18:11.939+0100.
        date_modified (Union[Unset, str]):  Example: 2021-12-31 20:18:11.939+0100.
        utc_date_created (Union[Unset, str]):  Example: 2021-12-31 19:18:11.939000+00:00.
        utc_date_modified (Union[Unset, str]):  Example: 2021-12-31 19:18:11.939000+00:00.
    """

    note_id: Union[Unset, str] = UNSET
    title: Union[Unset, str] = UNSET
    type: Union[Unset, NoteType] = UNSET
    mime: Union[Unset, str] = UNSET
    is_protected: Union[Unset, bool] = UNSET
    attributes: Union[Unset, List["Attribute"]] = UNSET
    parent_note_ids: Union[Unset, List[str]] = UNSET
    child_note_ids: Union[Unset, List[str]] = UNSET
    parent_branch_ids: Union[Unset, List[str]] = UNSET
    child_branch_ids: Union[Unset, List[str]] = UNSET
    date_created: Union[Unset, str] = UNSET
    date_modified: Union[Unset, str] = UNSET
    utc_date_created: Union[Unset, str] = UNSET
    utc_date_modified: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        note_id = self.note_id
        title = self.title
        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        mime = self.mime
        is_protected = self.is_protected
        attributes: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.attributes, Unset):
            attributes = []
            for componentsschemas_attribute_list_item_data in self.attributes:
                componentsschemas_attribute_list_item = componentsschemas_attribute_list_item_data.to_dict()

                attributes.append(componentsschemas_attribute_list_item)

        parent_note_ids: Union[Unset, List[str]] = UNSET
        if not isinstance(self.parent_note_ids, Unset):
            parent_note_ids = self.parent_note_ids

        child_note_ids: Union[Unset, List[str]] = UNSET
        if not isinstance(self.child_note_ids, Unset):
            child_note_ids = self.child_note_ids

        parent_branch_ids: Union[Unset, List[str]] = UNSET
        if not isinstance(self.parent_branch_ids, Unset):
            parent_branch_ids = self.parent_branch_ids

        child_branch_ids: Union[Unset, List[str]] = UNSET
        if not isinstance(self.child_branch_ids, Unset):
            child_branch_ids = self.child_branch_ids

        date_created = self.date_created
        date_modified = self.date_modified
        utc_date_created = self.utc_date_created
        utc_date_modified = self.utc_date_modified

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if note_id is not UNSET:
            field_dict["noteId"] = note_id
        if title is not UNSET:
            field_dict["title"] = title
        if type is not UNSET:
            field_dict["type"] = type
        if mime is not UNSET:
            field_dict["mime"] = mime
        if is_protected is not UNSET:
            field_dict["isProtected"] = is_protected
        if attributes is not UNSET:
            field_dict["attributes"] = attributes
        if parent_note_ids is not UNSET:
            field_dict["parentNoteIds"] = parent_note_ids
        if child_note_ids is not UNSET:
            field_dict["childNoteIds"] = child_note_ids
        if parent_branch_ids is not UNSET:
            field_dict["parentBranchIds"] = parent_branch_ids
        if child_branch_ids is not UNSET:
            field_dict["childBranchIds"] = child_branch_ids
        if date_created is not UNSET:
            field_dict["dateCreated"] = date_created
        if date_modified is not UNSET:
            field_dict["dateModified"] = date_modified
        if utc_date_created is not UNSET:
            field_dict["utcDateCreated"] = utc_date_created
        if utc_date_modified is not UNSET:
            field_dict["utcDateModified"] = utc_date_modified

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.attribute import Attribute

        d = src_dict.copy()
        note_id = d.pop("noteId", UNSET)

        title = d.pop("title", UNSET)

        _type = d.pop("type", UNSET)
        type: Union[Unset, NoteType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = NoteType(_type)

        mime = d.pop("mime", UNSET)

        is_protected = d.pop("isProtected", UNSET)

        attributes = []
        _attributes = d.pop("attributes", UNSET)
        for componentsschemas_attribute_list_item_data in _attributes or []:
            componentsschemas_attribute_list_item = Attribute.from_dict(componentsschemas_attribute_list_item_data)

            attributes.append(componentsschemas_attribute_list_item)

        parent_note_ids = cast(List[str], d.pop("parentNoteIds", UNSET))

        child_note_ids = cast(List[str], d.pop("childNoteIds", UNSET))

        parent_branch_ids = cast(List[str], d.pop("parentBranchIds", UNSET))

        child_branch_ids = cast(List[str], d.pop("childBranchIds", UNSET))

        date_created = d.pop("dateCreated", UNSET)

        date_modified = d.pop("dateModified", UNSET)

        utc_date_created = d.pop("utcDateCreated", UNSET)

        utc_date_modified = d.pop("utcDateModified", UNSET)

        note = cls(
            note_id=note_id,
            title=title,
            type=type,
            mime=mime,
            is_protected=is_protected,
            attributes=attributes,
            parent_note_ids=parent_note_ids,
            child_note_ids=child_note_ids,
            parent_branch_ids=parent_branch_ids,
            child_branch_ids=child_branch_ids,
            date_created=date_created,
            date_modified=date_modified,
            utc_date_created=utc_date_created,
            utc_date_modified=utc_date_modified,
        )

        note.additional_properties = d
        return note

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class NoteType(str, Enum):
    BOOK = "book"
    CODE = "code"
    FILE = "file"
    IMAGE = "image"
    MERMAID = "mermaid"
    NOTEMAP = "noteMap"
    RELATIONMAP = "relationMap"
    RENDER = "render"
    SEARCH = "search"
    SHORTCUT = "shortcut"
    TEXT = "text"
    WEBVIEW = "webView"

    def __str__(self) -> str:
        return str(self.value)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.note import Note
    from ..models.search_response_debug_info import SearchResponseDebugInfo


T = TypeVar("T", bound="SearchResponse")


@attr.s(auto_attribs=True)
class SearchResponse:
    """
    Attributes:
        results (List['Note']):
        debug_info (Union[Unset, SearchResponseDebugInfo]): debugging info on parsing the search query enabled with
            &debug=true parameter
    """

    results: List["Note"]
    debug_info: Union[Unset, "SearchResponseDebugInfo"] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        results = []
        for results_item_data in self.results:
            results_item = results_item_data.to_dict()

            results.append(results_item)

        debug_info: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.debug_info, Unset):
            debug_info = self.debug_info.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "results": results,
            }
        )
        if debug_info is not UNSET:
            field_dict["debugInfo"] = debug_info

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.note import Note
        from ..models.search_response_debug_info import SearchResponseDebugInfo

        d = src_dict.copy()
        results = []
        _results = d.pop("results")
        for results_item_data in _results:
            results_item = Note.from_dict(results_item_data)

            results.append(results_item)

        _debug_info = d.pop("debugInfo", UNSET)
        debug_info: Union[Unset, SearchResponseDebugInfo]
        if isinstance(_debug_info, Unset):
            debug_info = UNSET
        else:
            debug_info = SearchResponseDebugInfo.from_dict(_debug_info)

        search_response = cls(
            results=results,
            debug_info=debug_info,
        )

        search_response.additional_properties = d
        return search_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



""" Contains all the data models used in inputs/outputs """

from .app_info import AppInfo
from .attribute import Attribute
from .attribute_type import AttributeType
from .branch import Branch
from .create_note_def import CreateNoteDef
from .create_note_def_type import CreateNoteDefType
from .create_note_response_201 import CreateNoteResponse201
from .create_note_revision_format import CreateNoteRevisionFormat
from .error import Error
from .export_note_subtree_format import ExportNoteSubtreeFormat
from .login_json_body import LoginJsonBody
from .login_response_201 import LoginResponse201
from .note import Note
from .note_type import NoteType
from .search_notes_order_direction import SearchNotesOrderDirection
from .search_response import SearchResponse
from .search_response_debug_info import SearchResponseDebugInfo

__all__ = (
    "AppInfo",
    "Attribute",
    "AttributeType",
    "Branch",
    "CreateNoteDef",
    "CreateNoteDefType",
    "CreateNoteResponse201",
    "CreateNoteRevisionFormat",
    "Error",
    "ExportNoteSubtreeFormat",
    "LoginJsonBody",
    "LoginResponse201",
    "Note",
    "NoteType",
    "SearchNotesOrderDirection",
    "SearchResponse",
    "SearchResponseDebugInfo",
)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.branch import Branch
    from ..models.note import Note


T = TypeVar("T", bound="CreateNoteResponse201")


@attr.s(auto_attribs=True)
class CreateNoteResponse201:
    """
    Attributes:
        note (Union[Unset, Note]):
        branch (Union[Unset, Branch]): Branch places the note into the tree, it represents the relationship between a
            parent note and child note
    """

    note: Union[Unset, "Note"] = UNSET
    branch: Union[Unset, "Branch"] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        note: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.note, Unset):
            note = self.note.to_dict()

        branch: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.branch, Unset):
            branch = self.branch.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if note is not UNSET:
            field_dict["note"] = note
        if branch is not UNSET:
            field_dict["branch"] = branch

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.branch import Branch
        from ..models.note import Note

        d = src_dict.copy()
        _note = d.pop("note", UNSET)
        note: Union[Unset, Note]
        if isinstance(_note, Unset):
            note = UNSET
        else:
            note = Note.from_dict(_note)

        _branch = d.pop("branch", UNSET)
        branch: Union[Unset, Branch]
        if isinstance(_branch, Unset):
            branch = UNSET
        else:
            branch = Branch.from_dict(_branch)

        create_note_response_201 = cls(
            note=note,
            branch=branch,
        )

        create_note_response_201.additional_properties = d
        return create_note_response_201

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import Client
from ...models.search_notes_order_direction import SearchNotesOrderDirection
from ...models.search_response import SearchResponse
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    client: Client,
    search: str,
    fast_search: Union[Unset, None, bool] = False,
    include_archived_notes: Union[Unset, None, bool] = False,
    ancestor_note_id: Union[Unset, None, str] = UNSET,
    ancestor_depth: Union[Unset, None, str] = UNSET,
    order_by: Union[Unset, None, str] = UNSET,
    order_direction: Union[Unset, None, SearchNotesOrderDirection] = SearchNotesOrderDirection.ASC,
    limit: Union[Unset, None, int] = UNSET,
    debug: Union[Unset, None, bool] = False,
) -> Dict[str, Any]:
    url = "{}/notes".format(client.base_url)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    params["search"] = search

    params["fastSearch"] = fast_search

    params["includeArchivedNotes"] = include_archived_notes

    params["ancestorNoteId"] = ancestor_note_id

    params["ancestorDepth"] = ancestor_depth

    params["orderBy"] = order_by

    json_order_direction: Union[Unset, None, str] = UNSET
    if not isinstance(order_direction, Unset):
        json_order_direction = order_direction.value if order_direction else None

    params["orderDirection"] = json_order_direction

    params["limit"] = limit

    params["debug"] = debug

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(*, client: Client, response: httpx.Response) -> Optional[SearchResponse]:
    if response.status_code == HTTPStatus.OK:
        response_200 = SearchResponse.from_dict(response.json())

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(f"Unexpected status code: {response.status_code}")
    else:
        return None


def _build_response(*, client: Client, response: httpx.Response) -> Response[SearchResponse]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: Client,
    search: str,
    fast_search: Union[Unset, None, bool] = False,
    include_archived_notes: Union[Unset, None, bool] = False,
    ancestor_note_id: Union[Unset, None, str] = UNSET,
    ancestor_depth: Union[Unset, None, str] = UNSET,
    order_by: Union[Unset, None, str] = UNSET,
    order_direction: Union[Unset, None, SearchNotesOrderDirection] = SearchNotesOrderDirection.ASC,
    limit: Union[Unset, None, int] = UNSET,
    debug: Union[Unset, None, bool] = False,
) -> Response[SearchResponse]:
    """Search notes

    Args:
        search (str):
        fast_search (Union[Unset, None, bool]):
        include_archived_notes (Union[Unset, None, bool]):
        ancestor_note_id (Union[Unset, None, str]):  Example: evnnmvHTCgIn.
        ancestor_depth (Union[Unset, None, str]):
        order_by (Union[Unset, None, str]):
        order_direction (Union[Unset, None, SearchNotesOrderDirection]):  Default:
            SearchNotesOrderDirection.ASC.
        limit (Union[Unset, None, int]):
        debug (Union[Unset, None, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[SearchResponse]
    """

    kwargs = _get_kwargs(
        client=client,
        search=search,
        fast_search=fast_search,
        include_archived_notes=include_archived_notes,
        ancestor_note_id=ancestor_note_id,
        ancestor_depth=ancestor_depth,
        order_by=order_by,
        order_direction=order_direction,
        limit=limit,
        debug=debug,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: Client,
    search: str,
    fast_search: Union[Unset, None, bool] = False,
    include_archived_notes: Union[Unset, None, bool] = False,
    ancestor_note_id: Union[Unset, None, str] = UNSET,
    ancestor_depth: Union[Unset, None, str] = UNSET,
    order_by: Union[Unset, None, str] = UNSET,
    order_direction: Union[Unset, None, SearchNotesOrderDirection] = SearchNotesOrderDirection.ASC,
    limit: Union[Unset, None, int] = UNSET,
    debug: Union[Unset, None, bool] = False,
) -> Optional[SearchResponse]:
    """Search notes

    Args:
        search (str):
        fast_search (Union[Unset, None, bool]):
        include_archived_notes (Union[Unset, None, bool]):
        ancestor_note_id (Union[Unset, None, str]):  Example: evnnmvHTCgIn.
        ancestor_depth (Union[Unset, None, str]):
        order_by (Union[Unset, None, str]):
        order_direction (Union[Unset, None, SearchNotesOrderDirection]):  Default:
            SearchNotesOrderDirection.ASC.
        limit (Union[Unset, None, int]):
        debug (Union[Unset, None, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[SearchResponse]
    """

    return sync_detailed(
        client=client,
        search=search,
        fast_search=fast_search,
        include_archived_notes=include_archived_notes,
        ancestor_note_id=ancestor_note_id,
        ancestor_depth=ancestor_depth,
        order_by=order_by,
        order_direction=order_direction,
        limit=limit,
        debug=debug,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    search: str,
    fast_search: Union[Unset, None, bool] = False,
    include_archived_notes: Union[Unset, None, bool] = False,
    ancestor_note_id: Union[Unset, None, str] = UNSET,
    ancestor_depth: Union[Unset, None, str] = UNSET,
    order_by: Union[Unset, None, str] = UNSET,
    order_direction: Union[Unset, None, SearchNotesOrderDirection] = SearchNotesOrderDirection.ASC,
    limit: Union[Unset, None, int] = UNSET,
    debug: Union[Unset, None, bool] = False,
) -> Response[SearchResponse]:
    """Search notes

    Args:
        search (str):
        fast_search (Union[Unset, None, bool]):
        include_archived_notes (Union[Unset, None, bool]):
        ancestor_note_id (Union[Unset, None, str]):  Example: evnnmvHTCgIn.
        ancestor_depth (Union[Unset, None, str]):
        order_by (Union[Unset, None, str]):
        order_direction (Union[Unset, None, SearchNotesOrderDirection]):  Default:
            SearchNotesOrderDirection.ASC.
        limit (Union[Unset, None, int]):
        debug (Union[Unset, None, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[SearchResponse]
    """

    kwargs = _get_kwargs(
        client=client,
        search=search,
        fast_search=fast_search,
        include_archived_notes=include_archived_notes,
        ancestor_note_id=ancestor_note_id,
        ancestor_depth=ancestor_depth,
        order_by=order_by,
        order_direction=order_direction,
        limit=limit,
        debug=debug,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: Client,
    search: str,
    fast_search: Union[Unset, None, bool] = False,
    include_archived_notes: Union[Unset, None, bool] = False,
    ancestor_note_id: Union[Unset, None, str] = UNSET,
    ancestor_depth: Union[Unset, None, str] = UNSET,
    order_by: Union[Unset, None, str] = UNSET,
    order_direction: Union[Unset, None, SearchNotesOrderDirection] = SearchNotesOrderDirection.ASC,
    limit: Union[Unset, None, int] = UNSET,
    debug: Union[Unset, None, bool] = False,
) -> Optional[SearchResponse]:
    """Search notes

    Args:
        search (str):
        fast_search (Union[Unset, None, bool]):
        include_archived_notes (Union[Unset, None, bool]):
        ancestor_note_id (Union[Unset, None, str]):  Example: evnnmvHTCgIn.
        ancestor_depth (Union[Unset, None, str]):
        order_by (Union[Unset, None, str]):
        order_direction (Union[Unset, None, SearchNotesOrderDirection]):  Default:
            SearchNotesOrderDirection.ASC.
        limit (Union[Unset, None, int]):
        debug (Union[Unset, None, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[SearchResponse]
    """

    return (
        await asyncio_detailed(
            client=client,
            search=search,
            fast_search=fast_search,
            include_archived_notes=include_archived_notes,
            ancestor_note_id=ancestor_note_id,
            ancestor_depth=ancestor_depth,
            order_by=order_by,
            order_direction=order_direction,
            limit=limit,
            debug=debug,
        )
    ).parsed



from http import HTTPStatus
from typing import Any, Dict, Optional

import httpx

from ... import errors
from ...client import Client
from ...types import Response


def _get_kwargs(
    parent_note_id: str,
    *,
    client: Client,
) -> Dict[str, Any]:
    url = "{}/refresh-note-ordering/{parentNoteId}".format(client.base_url, parentNoteId=parent_note_id)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "method": "post",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
    }


def _parse_response(*, client: Client, response: httpx.Response) -> Optional[Any]:
    if response.status_code == HTTPStatus.NO_CONTENT:
        return None
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(f"Unexpected status code: {response.status_code}")
    else:
        return None


def _build_response(*, client: Client, response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    parent_note_id: str,
    *,
    client: Client,
) -> Response[Any]:
    r"""notePositions in branches are not automatically pushed to connected clients and need a specific
    instruction.  If you want your changes to be in effect immediately, call this service after setting
    branches' notePosition.  Note that you need to supply \"parentNoteId\" of branch(es) with changed
    positions.

    Args:
        parent_note_id (str):  Example: evnnmvHTCgIn.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        parent_note_id=parent_note_id,
        client=client,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(client=client, response=response)


async def asyncio_detailed(
    parent_note_id: str,
    *,
    client: Client,
) -> Response[Any]:
    r"""notePositions in branches are not automatically pushed to connected clients and need a specific
    instruction.  If you want your changes to be in effect immediately, call this service after setting
    branches' notePosition.  Note that you need to supply \"parentNoteId\" of branch(es) with changed
    positions.

    Args:
        parent_note_id (str):  Example: evnnmvHTCgIn.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        parent_note_id=parent_note_id,
        client=client,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(client=client, response=response)



from enum import Enum


class CreateNoteDefType(str, Enum):
    BOOK = "book"
    CODE = "code"
    FILE = "file"
    IMAGE = "image"
    RELATIONMAP = "relationMap"
    RENDER = "render"
    SEARCH = "search"
    TEXT = "text"

    def __str__(self) -> str:
        return str(self.value)
